﻿Option Strict On
Option Explicit On

Imports System.Threading
Imports System.Globalization
Imports System.Runtime.InteropServices

Public Class SpellChecker
    Implements ISpellCheckerChangedEventHandler

#Region "Enum"
    Private Enum CORRECTIVE_ACTION As Integer
        CORRECTIVE_ACTION_NONE = 0
        CORRECTIVE_ACTION_GET_SUGGESTIONS = 1
        CORRECTIVE_ACTION_REPLACE = 2
        CORRECTIVE_ACTION_DELETE = 3
    End Enum

    Private Enum WORDLIST_TYPE As Integer
        WORDLIST_TYPE_IGNORE = 0
        WORDLIST_TYPE_ADD = 1
        WORDLIST_TYPE_EXCLUDE = 2
        WORDLIST_TYPE_AUTOCORRECT = 3
    End Enum
#End Region

#Region "Const"
    Private Const S_OK As Integer = 0

    Private Const CLSID_SpellCheckerFactory As String = "7ab36653-1796-484b-bdfa-e74f1db7c1dc"

    Private Const IID_IEnumString As String = "00000101-0000-0000-C000-000000000046"
    Private Const IID_ISpellChecker As String = "b6fd0b71-e2bc-4653-8d05-f197e412770b"
    Private Const IID_ISpellChecker2 As String = "e7ed1c71-87f7-4378-a840-c9200dacee47"
    Private Const IID_ISpellingError As String = "b7c82d61-fbe8-4b47-9b27-6c0d2e0de0a3"
    Private Const IID_IEnumSpellingError As String = "803e3bd4-2828-4410-8290-418d1d73c762"
    Private Const IID_IOptionDescription As String = "432e5f85-35cf-4606-a801-6f70277e1d7a"
    Private Const IID_ISpellCheckerFactory As String = "8e018a9d-2415-4677-bf08-794ea61f94bb"
    Private Const IID_IUserDictionariesRegistrar As String = "aa176b85-0e12-4844-8e1a-eef1da77f586"
    Private Const IID_ISpellCheckerChangedEventHandler As String = "0b83a5b0-792f-4eab-9799-acf52c5ed08a"
#End Region

#Region "Class"
    Sub New()
        m_ISpellCheckerFactory = CType(Activator.CreateInstance(Type.GetTypeFromCLSID(New Guid(CLSID_SpellCheckerFactory))), ISpellCheckerFactory)
        If m_ISpellCheckerFactory IsNot Nothing Then
            Dim EnumString As IEnumString = Nothing
            If m_ISpellCheckerFactory.GetSupportedLanguages(EnumString) = S_OK Then
                If EnumString IsNot Nothing Then
                    Dim intCurrentLanguageIndex As Integer = 0
                    Dim strLanguageTag As String = String.Empty
                    Do While EnumString.Next(1, strLanguageTag, 0) = S_OK
                        m_SupportedLanguage.Add(New SUPPORTED_LANGUAGE(strLanguageTag, New CultureInfo(strLanguageTag).DisplayName))
                        If strLanguageTag.Equals(Thread.CurrentThread.CurrentCulture.Name) Then CurrentLanguageIndex = intCurrentLanguageIndex
                        intCurrentLanguageIndex += 1
                    Loop
                    Marshal.ReleaseComObject(EnumString)
                End If
            End If
        End If
    End Sub

    Protected Overrides Sub Finalize()
        If m_ISpellCheckerFactory IsNot Nothing Then
            If m_ISpellChecker IsNot Nothing Then
                If m_Cookie <> 0 Then
                    If m_ISpellChecker.RemoveSpellCheckerChanged(m_Cookie) = S_OK Then m_Cookie = 0
                End If
                If Marshal.ReleaseComObject(m_ISpellChecker) = 0 Then m_ISpellChecker = Nothing
            End If
            If Marshal.ReleaseComObject(m_ISpellCheckerFactory) = 0 Then m_ISpellCheckerFactory = Nothing
        End If
        MyBase.Finalize()
    End Sub
#End Region

#Region "Variable"
    Private m_Cookie As Integer = 0
    Private m_SpellCheckInfo As SPELLCHECK_INFO
    Private m_CurrentLanguageIndex As Integer = 0
    Private m_ISpellChecker As ISpellChecker = Nothing
    Private m_ISpellCheckerFactory As ISpellCheckerFactory = Nothing
    Private ReadOnly m_SupportedLanguage As New List(Of SUPPORTED_LANGUAGE)
#End Region

#Region "Structure"
    Private Structure SUPPORTED_LANGUAGE
        Dim LanguageTag As String
        Dim LanguageDisplayName As String
        Sub New(LanguageTag As String, LanguageDisplayName As String)
            Me.LanguageTag = LanguageTag
            Me.LanguageDisplayName = LanguageDisplayName
        End Sub
    End Structure

    Private Structure SPELLCHECK_INFO_OPTIONS
        Dim OptionValue As Byte
        Dim Id As String
        Dim Heading As String
        Dim Description As String
        Dim DescriptionLabel As List(Of String)
    End Structure

    Private Structure SPELLCHECK_INFO
        Dim LocalizedName As String
        Dim Id As String
        Dim LanguageTag As String
        Dim Options As List(Of SPELLCHECK_INFO_OPTIONS)
    End Structure
#End Region

#Region "Properties"
    Public Property CurrentLanguageIndex As Integer
        Get
            Return m_CurrentLanguageIndex
        End Get
        Set(value As Integer)
            If (value >= 0) And (value <= GetSupportedLanguageCount - 1) Then
                If value <> m_CurrentLanguageIndex Then
                    m_CurrentLanguageIndex = value
                    CreateSpellCheck(value)
                End If
            End If
        End Set
    End Property

    Public ReadOnly Property GetSupportedLanguageCount As Integer
        Get
            Return m_SupportedLanguage.Count
        End Get
    End Property

    Public ReadOnly Property GetSupportedLanguageTag(Index As Integer) As String
        Get
            Return m_SupportedLanguage(Index).LanguageTag
        End Get
    End Property

    Public ReadOnly Property GetSupportedLanguageDisplayName(Index As Integer) As String
        Get
            Return m_SupportedLanguage(Index).LanguageDisplayName
        End Get
    End Property

    Public ReadOnly Property GetCurrentSpellCheckLocalizedName() As String
        Get
            Return m_SpellCheckInfo.LocalizedName
        End Get
    End Property

    Public ReadOnly Property GetCurrentSpellCheckId() As String
        Get
            Return m_SpellCheckInfo.Id
        End Get
    End Property

    Public ReadOnly Property GetCurrentSpellCheckLanguageTag() As String
        Get
            Return m_SpellCheckInfo.LanguageTag
        End Get
    End Property


    Public ReadOnly Property GetCurrentSpellCheckOptionsCount() As Integer
        Get
            Return m_SpellCheckInfo.Options.Count
        End Get
    End Property

    Public ReadOnly Property GetCurrentSpellCheckOptionsValue(Index As Integer) As Byte
        Get
            Return m_SpellCheckInfo.Options(Index).OptionValue
        End Get
    End Property

    Public ReadOnly Property GetCurrentSpellCheckOptionsID(Index As Integer) As String
        Get
            Return m_SpellCheckInfo.Options(Index).Id
        End Get
    End Property

    Public ReadOnly Property GetCurrentSpellCheckOptionsHeading(Index As Integer) As String
        Get
            Return m_SpellCheckInfo.Options(Index).Heading
        End Get
    End Property

    Public ReadOnly Property GetCurrentSpellCheckOptionsDescription(Index As Integer) As String
        Get
            Return m_SpellCheckInfo.Options(Index).Description
        End Get
    End Property

    Public ReadOnly Property GetCurrentSpellCheckOptionsDescriptionLabelCount(Index As Integer) As Integer
        Get
            Return m_SpellCheckInfo.Options(Index).DescriptionLabel.Count
        End Get
    End Property

    Public ReadOnly Property GetCurrentSpellCheckOptionsDescriptionLabel(Index As Integer, LabelIndex As Integer) As String
        Get
            Return m_SpellCheckInfo.Options(Index).DescriptionLabel(LabelIndex)
        End Get
    End Property
#End Region

#Region "Public Subs"
    Public Sub CheckText(Text As String, Optional Comprehensive As Boolean = False)
        If Not String.IsNullOrEmpty(Text) Then
            If m_ISpellChecker IsNot Nothing Then
                Dim bolIsEnumSpellingError As Boolean = True
                Dim EnumSpellingError As IEnumSpellingError = Nothing
                If Comprehensive Then
                    If m_ISpellChecker.ComprehensiveCheck(Text, EnumSpellingError) = S_OK Then
                        If EnumSpellingError IsNot Nothing Then bolIsEnumSpellingError = True
                    End If
                Else
                    If m_ISpellChecker.Check(Text, EnumSpellingError) = S_OK Then
                        If EnumSpellingError IsNot Nothing Then bolIsEnumSpellingError = True
                    End If
                End If
                If bolIsEnumSpellingError Then
                    Dim SpellingError As ISpellingError = Nothing
                    Do While EnumSpellingError.Next(SpellingError) = S_OK
                        If SpellingError IsNot Nothing Then
                            Dim intStart As Integer = 0
                            Dim intLenght As Integer = 0
                            If SpellingError.GetStartIndex(intStart) = S_OK Then
                                If SpellingError.GetLength(intLenght) = S_OK Then
                                    Dim strErrWord As String = Text.Substring(intStart, intLenght)
                                    Dim eAction As CORRECTIVE_ACTION = CORRECTIVE_ACTION.CORRECTIVE_ACTION_NONE
                                    If SpellingError.GetCorrectiveAction(eAction) = S_OK Then
                                        If eAction <> CORRECTIVE_ACTION.CORRECTIVE_ACTION_NONE Then
                                            Select Case eAction
                                                Case CORRECTIVE_ACTION.CORRECTIVE_ACTION_GET_SUGGESTIONS
                                                    Dim EnumString As IEnumString = Nothing
                                                    If m_ISpellChecker.Suggest(strErrWord, EnumString) = S_OK Then
                                                        If EnumString IsNot Nothing Then
                                                            Dim strList As New List(Of String)
                                                            Dim strSuggestion As String = String.Empty
                                                            Do While EnumString.Next(1, strSuggestion, 0) = S_OK
                                                                strList.Add(strSuggestion)
                                                                strSuggestion = String.Empty
                                                            Loop
                                                            Debug.Print("Replace Word '" & strErrWord & "' with Suggestions: " & String.Join(", ", strList.ToArray))
                                                            Marshal.ReleaseComObject(EnumString)
                                                        End If
                                                    End If
                                                Case CORRECTIVE_ACTION.CORRECTIVE_ACTION_REPLACE
                                                    Dim strReplace As String = String.Empty
                                                    If SpellingError.GetReplacement(strReplace) = S_OK Then
                                                        Debug.Print("Replace Word '" & strErrWord & "' with " & strReplace)
                                                    End If
                                                Case CORRECTIVE_ACTION.CORRECTIVE_ACTION_DELETE
                                                    Debug.Print("Double Word '" & strErrWord & "' can delete.")
                                            End Select
                                        End If
                                    End If
                                End If
                            End If
                            Marshal.ReleaseComObject(SpellingError)
                        End If
                    Loop
                    Marshal.ReleaseComObject(EnumSpellingError)
                End If
            End If
        End If
    End Sub
#End Region

#Region "Private Subs"
    Private Function CreateSpellCheck(Index As Integer) As Boolean
        Dim bolRet As Boolean = False
        If m_ISpellCheckerFactory IsNot Nothing Then
            If m_ISpellChecker IsNot Nothing Then
                If m_Cookie <> 0 Then
                    If m_ISpellChecker.RemoveSpellCheckerChanged(m_Cookie) = S_OK Then m_Cookie = 0
                End If
                If Marshal.ReleaseComObject(m_ISpellChecker) = 0 Then m_ISpellChecker = Nothing
            End If
            Dim bolIsSupported As Boolean = False
            Dim strLanguageTag As String = m_SupportedLanguage(Index).LanguageTag
            If m_ISpellCheckerFactory.IsSupported(strLanguageTag, bolIsSupported) = S_OK Then
                If bolIsSupported Then
                    If m_ISpellCheckerFactory.CreateSpellChecker(strLanguageTag, m_ISpellChecker) = S_OK Then
                        If m_ISpellChecker IsNot Nothing Then
                            bolRet = True
                            Dim tSpellCheckInfo As New SPELLCHECK_INFO With {.Options = New List(Of SPELLCHECK_INFO_OPTIONS)}
                            If m_ISpellChecker.AddSpellCheckerChanged(Me, m_Cookie) = S_OK Then
                                m_ISpellChecker.GetLocalizedName(tSpellCheckInfo.LocalizedName)
                                m_ISpellChecker.GetId(tSpellCheckInfo.Id)
                                m_ISpellChecker.GetLanguageTag(tSpellCheckInfo.LanguageTag)
                                Dim EnumString As IEnumString = Nothing
                                If m_ISpellChecker.GetOptionIds(EnumString) = S_OK Then
                                    If EnumString IsNot Nothing Then
                                        Dim strOptions As String = String.Empty
                                        Do While EnumString.Next(1, strOptions, 0) = S_OK
                                            Dim tSPELLCHECK_INFO_OPTIONS As New SPELLCHECK_INFO_OPTIONS With {.DescriptionLabel = New List(Of String)}
                                            m_ISpellChecker.GetOptionValue(strOptions, tSPELLCHECK_INFO_OPTIONS.OptionValue)
                                            Dim OptionDescription As IOptionDescription = Nothing
                                            If m_ISpellChecker.GetOptionDescription(strOptions, OptionDescription) = S_OK Then
                                                If OptionDescription IsNot Nothing Then
                                                    OptionDescription.GetId(tSPELLCHECK_INFO_OPTIONS.Id)
                                                    OptionDescription.GetHeading(tSPELLCHECK_INFO_OPTIONS.Heading)
                                                    OptionDescription.GetDescription(tSPELLCHECK_INFO_OPTIONS.Description)
                                                    Dim EnumString2 As IEnumString = Nothing
                                                    If OptionDescription.GetLabels(EnumString2) = S_OK Then
                                                        If EnumString2 IsNot Nothing Then
                                                            Dim strLabel As String = String.Empty
                                                            Do While EnumString2.Next(1, strLabel, 0) = S_OK
                                                                tSPELLCHECK_INFO_OPTIONS.DescriptionLabel.Add(strLabel)
                                                                strLabel = String.Empty
                                                            Loop
                                                            Marshal.ReleaseComObject(EnumString2)
                                                        End If
                                                    End If
                                                    Marshal.ReleaseComObject(OptionDescription)
                                                End If
                                            End If
                                            strOptions = String.Empty
                                            tSpellCheckInfo.Options.Add(tSPELLCHECK_INFO_OPTIONS)
                                        Loop
                                        Marshal.ReleaseComObject(EnumString)
                                    End If
                                End If
                            End If
                            m_SpellCheckInfo = tSpellCheckInfo
                        End If
                    End If
                End If
            End If
        End If
        Return bolRet
    End Function
#End Region

#Region "Interface IEnumString"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IEnumString)>
    Private Interface IEnumString
        <PreserveSig> Function [Next](<[In]> celt As Integer,
                                      <Out, MarshalAs(UnmanagedType.LPWStr)> ByRef rgelt As String,
                                      <Out> ByRef pceltFetched As Integer) As Integer
        <PreserveSig> Function Skip(<[In]> celt As Integer) As Integer
        <PreserveSig> Function Reset() As Integer
        <PreserveSig> Function Clone(<Out, MarshalAs(UnmanagedType.Interface)> ByRef ppenum As IEnumString) As Integer
    End Interface
#End Region

#Region "Interface ISpellChecker"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_ISpellChecker)>
    Private Interface ISpellChecker
        <PreserveSig> Function GetLanguageTag(<Out, MarshalAs(UnmanagedType.LPWStr)> ByRef value As String) As Integer
        <PreserveSig> Function Check(<[In], MarshalAs(UnmanagedType.LPWStr)> text As String,
                                     <Out, MarshalAs(UnmanagedType.Interface)> ByRef value As IEnumSpellingError) As Integer
        <PreserveSig> Function Suggest(<[In], MarshalAs(UnmanagedType.LPWStr)> word As String,
                                       <Out, MarshalAs(UnmanagedType.Interface)> ByRef value As IEnumString) As Integer
        <PreserveSig> Function Add(<[In], MarshalAs(UnmanagedType.LPWStr)> word As String) As Integer
        <PreserveSig> Function Ignore(<[In], MarshalAs(UnmanagedType.LPWStr)> word As String) As Integer
        <PreserveSig> Function AutoCorrect(<[In], MarshalAs(UnmanagedType.LPWStr)> from As String,
                                           <[In], MarshalAs(UnmanagedType.LPWStr)> [to] As String) As Integer
        <PreserveSig> Function GetOptionValue(<[In], MarshalAs(UnmanagedType.LPWStr)> optionID As String,
                                              <Out> ByRef value As Byte) As Integer
        <PreserveSig> Function GetOptionIds(<Out, MarshalAs(UnmanagedType.Interface)> ByRef value As IEnumString) As Integer
        <PreserveSig> Function GetId(<Out, MarshalAs(UnmanagedType.LPWStr)> ByRef value As String) As Integer
        <PreserveSig> Function GetLocalizedName(<Out, MarshalAs(UnmanagedType.LPWStr)> ByRef value As String) As Integer
        <PreserveSig> Function AddSpellCheckerChanged(<[In], MarshalAs(UnmanagedType.Interface)> handler As ISpellCheckerChangedEventHandler,
                                                      <Out> ByRef eventCookie As Integer) As Integer
        <PreserveSig> Function RemoveSpellCheckerChanged(<[In]> eventCookie As Integer) As Integer
        <PreserveSig> Function GetOptionDescription(<[In], MarshalAs(UnmanagedType.LPWStr)> optionID As String,
                                                    <Out, MarshalAs(UnmanagedType.Interface)> ByRef value As IOptionDescription) As Integer
        <PreserveSig> Function ComprehensiveCheck(<[In], MarshalAs(UnmanagedType.LPWStr)> text As String,
                                                  <Out, MarshalAs(UnmanagedType.Interface)> ByRef value As IEnumSpellingError) As Integer
    End Interface
#End Region

#Region "Interface ISpellChecker2"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_ISpellChecker2)>
    Private Interface ISpellChecker2
        Inherits ISpellChecker

        ' ---=== ISpellChecker ===---

        <PreserveSig> Shadows Function GetLanguageTag(<Out, MarshalAs(UnmanagedType.LPWStr)> ByRef value As String) As Integer
        <PreserveSig> Shadows Function Check(<[In], MarshalAs(UnmanagedType.LPWStr)> text As String,
                                             <Out, MarshalAs(UnmanagedType.Interface)> ByRef value As IEnumSpellingError) As Integer
        <PreserveSig> Shadows Function Suggest(<[In], MarshalAs(UnmanagedType.LPWStr)> word As String,
                                               <Out, MarshalAs(UnmanagedType.Interface)> ByRef value As IEnumString) As Integer
        <PreserveSig> Shadows Function Add(<[In], MarshalAs(UnmanagedType.LPWStr)> word As String) As Integer
        <PreserveSig> Shadows Function Ignore(<[In], MarshalAs(UnmanagedType.LPWStr)> word As String) As Integer
        <PreserveSig> Shadows Function AutoCorrect(<[In], MarshalAs(UnmanagedType.LPWStr)> from As String,
                                                   <[In], MarshalAs(UnmanagedType.LPWStr)> [to] As String) As Integer
        <PreserveSig> Shadows Function GetOptionValue(<[In], MarshalAs(UnmanagedType.LPWStr)> optionID As String,
                                                      <Out> ByRef value As Byte) As Integer
        <PreserveSig> Shadows Function GetOptionIds(<Out, MarshalAs(UnmanagedType.Interface)> ByRef value As IEnumString) As Integer
        <PreserveSig> Shadows Function GetId(<Out, MarshalAs(UnmanagedType.LPWStr)> ByRef value As String) As Integer
        <PreserveSig> Shadows Function GetLocalizedName(<Out, MarshalAs(UnmanagedType.LPWStr)> ByRef value As String) As Integer
        <PreserveSig> Shadows Function AddSpellCheckerChanged(<[In], MarshalAs(UnmanagedType.Interface)> handler As ISpellCheckerChangedEventHandler,
                                                              <Out> ByRef eventCookie As Integer) As Integer
        <PreserveSig> Shadows Function RemoveSpellCheckerChanged(<[In]> eventCookie As Integer) As Integer
        <PreserveSig> Shadows Function GetOptionDescription(<[In], MarshalAs(UnmanagedType.LPWStr)> optionID As String,
                                                            <Out, MarshalAs(UnmanagedType.Interface)> ByRef value As IOptionDescription) As Integer
        <PreserveSig> Shadows Function ComprehensiveCheck(<[In], MarshalAs(UnmanagedType.LPWStr)> text As String,
                                                          <Out, MarshalAs(UnmanagedType.Interface)> ByRef value As IEnumSpellingError) As Integer

        ' ---=== ISpellChecker2 ===---
        <PreserveSig> Function Remove(<[In], MarshalAs(UnmanagedType.LPWStr)> word As String) As Integer
    End Interface
#End Region

#Region "Interface ISpellingError"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_ISpellingError)>
    Private Interface ISpellingError
        <PreserveSig> Function GetStartIndex(<Out> ByRef value As Integer) As Integer
        <PreserveSig> Function GetLength(<Out> ByRef value As Integer) As Integer
        <PreserveSig> Function GetCorrectiveAction(<Out> ByRef value As CORRECTIVE_ACTION) As Integer
        <PreserveSig> Function GetReplacement(<Out, MarshalAs(UnmanagedType.LPWStr)> ByRef value As String) As Integer
    End Interface
#End Region

#Region "Interface IOptionDescription"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IOptionDescription)>
    Private Interface IOptionDescription
        <PreserveSig> Function GetId(<Out, MarshalAs(UnmanagedType.LPWStr)> ByRef value As String) As Integer
        <PreserveSig> Function GetHeading(<Out, MarshalAs(UnmanagedType.LPWStr)> ByRef value As String) As Integer
        <PreserveSig> Function GetDescription(<Out, MarshalAs(UnmanagedType.LPWStr)> ByRef value As String) As Integer
        <PreserveSig> Function GetLabels(<Out, MarshalAs(UnmanagedType.Interface)> ByRef value As IEnumString) As Integer
    End Interface
#End Region

#Region "Interface IEnumSpellingError"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IEnumSpellingError)>
    Private Interface IEnumSpellingError
        <PreserveSig> Function [Next](<Out, MarshalAs(UnmanagedType.Interface)> ByRef value As ISpellingError) As Integer
    End Interface
#End Region

#Region "Interface ISpellCheckerFactory"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_ISpellCheckerFactory)>
    Private Interface ISpellCheckerFactory
        <PreserveSig> Function GetSupportedLanguages(<Out, MarshalAs(UnmanagedType.Interface)> ByRef value As IEnumString) As Integer
        <PreserveSig> Function IsSupported(<[In], MarshalAs(UnmanagedType.LPWStr)> languageTag As String,
                                           <Out, MarshalAs(UnmanagedType.Bool)> ByRef value As Boolean) As Integer
        <PreserveSig> Function CreateSpellChecker(<[In], MarshalAs(UnmanagedType.LPWStr)> languageTag As String,
                                                  <Out, MarshalAs(UnmanagedType.Interface)> ByRef value As ISpellChecker) As Integer
    End Interface
#End Region

#Region "Interface IUserDictionariesRegistrar"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IUserDictionariesRegistrar)>
    Private Interface IUserDictionariesRegistrar
        <PreserveSig> Function RegisterUserDictionary(<[In], MarshalAs(UnmanagedType.LPWStr)> dictionaryPath As String,
                                                      <[In], MarshalAs(UnmanagedType.LPWStr)> languageTag As String) As Integer
        <PreserveSig> Function UnregisterUserDictionary(<[In], MarshalAs(UnmanagedType.LPWStr)> dictionaryPath As String,
                                                        <[In], MarshalAs(UnmanagedType.LPWStr)> languageTag As String) As Integer
    End Interface
#End Region

#Region "Interface ISpellCheckerChangedEventHandler"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_ISpellCheckerChangedEventHandler)>
    Private Interface ISpellCheckerChangedEventHandler
        <PreserveSig> Function Invoke(<[In], MarshalAs(UnmanagedType.Interface)> sender As ISpellChecker) As Integer
    End Interface
#End Region

#Region "Implements ISpellCheckerChangedEventHandler"
    Private Function Invoke(<[In]> <MarshalAs(UnmanagedType.Interface)> sender As ISpellChecker) As Integer Implements ISpellCheckerChangedEventHandler.Invoke
        Debug.Print("ISpellCheckerChangedEventHandler::Invoke")
        Return S_OK
    End Function
#End Region

End Class
